@@ -14,7 +14,5 @@ print "ok 1\n";
 
 ######################### End of black magic.
 
-# Insert your test code below (better if it prints "ok 13"
-# (correspondingly "not ok 13") depending on the success of chunk 13
-# of the test code):
-
+$x = Crypt::Random::makerandom ( Size => 10, Strength => 0, Method => "BV");
+print qq{$x\n};
